package org.tekkotsu.ui.editor.actions;

import java.util.Iterator;
import java.util.List;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.requests.CreateRequest;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.tekkotsu.ui.editor.StateMachineEditor;
import org.tekkotsu.ui.editor.model.ModelData;
import org.tekkotsu.ui.editor.model.SourceNodeModel;
import org.tekkotsu.ui.editor.model.SourceTransitionModel;
import org.tekkotsu.ui.util.Debugger;

/**
 * This action automatically generate layout for all node and transition
 */
public class GenerateLayoutAction extends Action {
	private StateMachineEditor editor;
	private ModelData model;
	private CreateRequest request;

	public GenerateLayoutAction(StateMachineEditor editor, ModelData model) {
		super("Generate layout");
		this.editor = editor;
		this.model = model;
		request = new CreateRequest();
	}

	public ModelData getModel() {
		return model;
	}

	public CreateRequest getRequest() {
		return request;
	}

	private void createLayout() {
		Point curPoint = request.getLocation();
		
		int radius;
		int w    = editor.getContainer().getBounds().width - 120;
		int h    = editor.getContainer().getBounds().height - 60;
		int wmin = w/2;
		int hmin = h/2;
		if (wmin<hmin)
			radius = wmin;
		else
			radius = hmin;
		
		radius *= 0.8;
		
		curPoint = new Point(wmin-60,hmin-30);

		int count = 0;
		
		// create nodes
		{
			List<SourceNodeModel> nodeList = model.getNodes();
			Iterator<SourceNodeModel> iter = nodeList.iterator();
			
			final int SIZE = nodeList.size()+1;
			
			while (iter.hasNext()) {
				SourceNodeModel node = iter.next();
				CreateNodeAction genNode = new CreateNodeAction(editor,node);
				int dx = (int)Math.round(radius*Math.cos(2*Math.PI/SIZE*count));
				int dy = (int)Math.round(radius*Math.sin(2*Math.PI/SIZE*count));
				genNode.getRequest().setLocation(curPoint.getTranslated(dx,dy));
				
				genNode.run();
				count++;
			}
		}
		
		//create transitions
		{
			List<SourceTransitionModel>     transList = model.getTransitions();
			Iterator<SourceTransitionModel> iter      = transList.iterator();
			while (iter.hasNext()) {
				SourceTransitionModel trans = iter.next();
				CreateTransitionAction genTrans = new CreateTransitionAction(editor,trans);
				genTrans.getRequest().setLocation(curPoint);
				curPoint.translate(40,0);
				genTrans.run();
			}
		}
	}

	@Override
	public void run() {
		//EditDomain domain = getEditDomain();
		//CommandStack stack = domain.getCommandStack();
		GraphicalViewer view = editor.getGraphicalViewer();
		
		try
		{
			SourceNodeModel nodeModel = model.getNodes().get(0);
			request.setFactory(nodeModel.getFactory());

			Debugger.printDebug(Debugger.DEBUG_ALL,"Generate entire layout.");
			createLayout();
			editor.setDirty();
		}
		catch (Exception e)
		{
			MessageDialog.openError(
				view.getControl().getShell(),
				"Error: no model loaded",
				"An exception occurred while trying to generate a layout for this model."
			);
			Debugger.printThrowable(e);
		}
	}
}