package org.tekkotsu.ui.rcp;

import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.ICoolBarManager;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.action.ToolBarContributionItem;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.ui.IPageLayout;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.actions.ContributionItemFactory;
import org.eclipse.ui.actions.ActionFactory.IWorkbenchAction;
import org.eclipse.ui.application.ActionBarAdvisor;
import org.eclipse.ui.application.IActionBarConfigurer;
import org.eclipse.ui.console.IConsoleConstants;
import org.tekkotsu.ui.rcp.actions.ShowActionFactory;

public class StoryboardActionBarAdvisor extends ActionBarAdvisor {

	// generic actions
	private IWorkbenchAction closeAction;
	private IWorkbenchAction closeAllAction;
	private IWorkbenchAction closeAllSavedAction;
	private IWorkbenchAction saveAction;
	private IWorkbenchAction saveAllAction;
	private IWorkbenchAction saveAsAction;
	
	// generic retarget actions
	private IWorkbenchAction undoAction;
	private IWorkbenchAction redoAction;
	private IWorkbenchAction cutAction;
	private IWorkbenchAction copyAction;
	private IWorkbenchAction pasteAction;
	private IWorkbenchAction selectAllAction;
	private IWorkbenchAction findAction;
	private IWorkbenchAction revertAction;
	private IWorkbenchAction quitAction;
	
	// show actions
	private IAction showStoryboardAction;
	private IAction showConsoleAction;
	private IAction showPropSheetAction;
	private IAction showRuntimeAction;
	private IAction showImageAction;
	
	// menu managers
	MenuManager editMenu;
	MenuManager fileMenu;
	MenuManager windowMenu;
	
	public StoryboardActionBarAdvisor(IActionBarConfigurer configurer) {
		super(configurer);
	}
	
	/**
	 * Creates actions (and contribution items) for the menu bar, toolbar and status line.
	 */
	@Override
	protected void makeActions(IWorkbenchWindow window) {

		IActionBarConfigurer abar = getActionBarConfigurer();
		
		// The actions in jface do not have menu vs. enable, vs. disable vs. color
		// There are actions in here being passed the workbench - problem 		
		saveAction = ActionFactory.SAVE.create(window);
		abar.registerGlobalAction(saveAction);

		saveAsAction = ActionFactory.SAVE_AS.create(window);
		abar.registerGlobalAction(saveAsAction);

		saveAllAction = ActionFactory.SAVE_ALL.create(window);
		abar.registerGlobalAction(saveAllAction);
		
		undoAction = ActionFactory.UNDO.create(window);
		abar.registerGlobalAction(undoAction);

		redoAction = ActionFactory.REDO.create(window);
		abar.registerGlobalAction(redoAction);

		cutAction = ActionFactory.CUT.create(window);
		abar.registerGlobalAction(cutAction);

		copyAction = ActionFactory.COPY.create(window);
		abar.registerGlobalAction(copyAction);

		pasteAction = ActionFactory.PASTE.create(window);
		abar.registerGlobalAction(pasteAction);

		selectAllAction = ActionFactory.SELECT_ALL.create(window);
		abar.registerGlobalAction(selectAllAction);
		
		findAction = ActionFactory.FIND.create(window);
		abar.registerGlobalAction(findAction);

		closeAction = ActionFactory.CLOSE.create(window);
		abar.registerGlobalAction(closeAction);

		closeAllAction = ActionFactory.CLOSE_ALL.create(window);
		abar.registerGlobalAction(closeAllAction);

		closeAllSavedAction = ActionFactory.CLOSE_ALL_SAVED.create(window);
		abar.registerGlobalAction(closeAllSavedAction);

		revertAction = ActionFactory.REVERT.create(window);
		abar.registerGlobalAction(revertAction);

		quitAction = ActionFactory.QUIT.create(window);
		abar.registerGlobalAction(quitAction);
		
		
		
		ShowActionFactory showFactory = new ShowActionFactory(window);
		
		showStoryboardAction = showFactory.create("Storyboard",StoryboardPerspective.ID_STORYBOARD_VIEW);
		abar.registerGlobalAction(showStoryboardAction);
		
		showConsoleAction = showFactory.create("Console",IConsoleConstants.ID_CONSOLE_VIEW);
		abar.registerGlobalAction(showConsoleAction);
		
		showPropSheetAction = showFactory.create("Property Sheet",IPageLayout.ID_PROP_SHEET);
		abar.registerGlobalAction(showPropSheetAction);
		
		showRuntimeAction = showFactory.create("Runtime Property",StoryboardPerspective.ID_RUNTIME_VIEW);
		abar.registerGlobalAction(showRuntimeAction);
		
		showImageAction = showFactory.create("Image Preview", StoryboardPerspective.ID_IMAGE_VIEW);
		abar.registerGlobalAction(showImageAction);
		
		windowMenu = createWindowMenu(window);
		editMenu   = createEditMenu(window);
		fileMenu   = createFileMenu(window);
	}
	
	@Override
	protected void fillCoolBar(ICoolBarManager cbManager) {
		cbManager.add(new GroupMarker("group.file")); //$NON-NLS-1$
		{ // File Group
			IToolBarManager fileToolBar = new ToolBarManager(cbManager.getStyle());
			fileToolBar.add(new Separator(IWorkbenchActionConstants.NEW_GROUP));
			fileToolBar.add(new GroupMarker(IWorkbenchActionConstants.SAVE_GROUP));
			fileToolBar.add(saveAction);
			
			fileToolBar.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
			
			// Add to the cool bar manager
			cbManager.add(new ToolBarContributionItem(fileToolBar,IWorkbenchActionConstants.TOOLBAR_FILE));
		}
		
		cbManager.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
		
		cbManager.add(new GroupMarker(IWorkbenchActionConstants.GROUP_EDITOR));
	}
	
	@Override
	protected void fillMenuBar(IMenuManager menubar) {
		menubar.add(fileMenu);
		menubar.add(editMenu);
		menubar.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
		menubar.add(windowMenu);
	}
	
	private MenuManager createWindowMenu(IWorkbenchWindow window) {
		MenuManager menu = new MenuManager("&Window",IWorkbenchActionConstants.M_WINDOW);		
		menu.add(showConsoleAction);		
		
		return menu;
	}
	
	/**
	 * Creates and returns the File menu.
	 */
	private MenuManager createFileMenu(IWorkbenchWindow window) {
		MenuManager menu = new MenuManager("&File", IWorkbenchActionConstants.M_FILE); //$NON-NLS-1$
		menu.add(new GroupMarker(IWorkbenchActionConstants.FILE_START));
		
		menu.add(new GroupMarker(IWorkbenchActionConstants.NEW_EXT));
		menu.add(closeAction);
		menu.add(closeAllAction);
		//		menu.add(closeAllSavedAction);
		menu.add(new GroupMarker(IWorkbenchActionConstants.CLOSE_EXT));
		menu.add(new Separator());
		menu.add(saveAction);
		menu.add(saveAsAction);
		menu.add(saveAllAction);

		menu.add(revertAction);
		menu.add(ContributionItemFactory.REOPEN_EDITORS.create(window));
		menu.add(new GroupMarker(IWorkbenchActionConstants.MRU));
		menu.add(new Separator());
		menu.add(quitAction);
		menu.add(new GroupMarker(IWorkbenchActionConstants.FILE_END));
		return menu;
	}

	/**
	 * Creates and returns the Edit menu.
	 */
	private MenuManager createEditMenu(IWorkbenchWindow window) {
		MenuManager menu = new MenuManager("&Edit", IWorkbenchActionConstants.M_EDIT); //$NON-NLS-1$
		menu.add(new GroupMarker(IWorkbenchActionConstants.EDIT_START));

		menu.add(undoAction);
		menu.add(redoAction);
		menu.add(new GroupMarker(IWorkbenchActionConstants.UNDO_EXT));

		menu.add(cutAction);
		menu.add(copyAction);
		menu.add(pasteAction);
		menu.add(new GroupMarker(IWorkbenchActionConstants.CUT_EXT));

		menu.add(selectAllAction);
		menu.add(new Separator());

		menu.add(findAction);
		menu.add(new GroupMarker(IWorkbenchActionConstants.FIND_EXT));

		menu.add(new GroupMarker(IWorkbenchActionConstants.ADD_EXT));

		menu.add(new GroupMarker(IWorkbenchActionConstants.EDIT_END));
		menu.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
		return menu;
	}
}
