package org.tekkotsu.ui.storyboard.objects;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.tekkotsu.ui.editor.model.StateNodeModel;
import org.tekkotsu.ui.storyboard.ResourceRegistry;
import org.tekkotsu.ui.storyboard.model.AbstractModel;
import org.tekkotsu.ui.storyboard.model.ImageModel;
import org.tekkotsu.ui.storyboard.panels.TimeLine;
import org.tekkotsu.ui.storyboard.views.RuntimeView;
import org.tekkotsu.ui.storyboard.views.StoryboardViewer;

/**
 * @author asangpet
 * 
 */
public class ImageViewObject extends AbstractViewObject implements
		PropertyChangeListener {
	ImageModel model;
	StateNodeModel state;

	public ImageViewObject(ImageModel model, StateNodeModel state,
			StoryboardViewer viewer) {
		super(viewer);
		this.state = state;
		setModel(model);
		setToolTip(getToolTipFigure());
	}

	private IFigure getToolTipFigure() {
		Label label = new Label("Record at:"
				+ RuntimeView.timeToString(model.getTime()));
		return label;
	}

	private String getImageName() {
		return model.getImageName();
	}

	private String getThumbImageName() {
		return "thumb_" + model.getID() + model.getTime();
	}
	
	private String getActiveThumbName() {
		return "active_" + getThumbImageName();
	}

	private void setModel(ImageModel model) {
		if (this.model != null) {
			this.model.removePropertyChangeListener(this);
		}
		this.model = model;
		model.addPropertyChangeListener(this);

		Rectangle bound = null;
		if (state == null)
			bound = new Rectangle(0, 6, 10, TimeLine.HEIGHT);
		else
			bound = state.getConstraint();

		int xoff = viewer.getTimeLine().getOffsetOfTime(model.getStart());

		ImageData act_tb = model.getImageData();
		act_tb.alpha = 255;
		
		final Image act_thumb = new Image(ResourceRegistry.getInstance()
				.getDisplay(), act_tb.scaledTo(48, 48));
		
		final Image image = new Image(ResourceRegistry.getInstance()
				.getDisplay(), model.getImageData());
		ImageData tb = model.getImageData();
		tb.alpha = 100;
		final Image thumb = new Image(ResourceRegistry.getInstance()
				.getDisplay(), tb.scaledTo(48, 48));
		
		tb.alpha = -1;

		if (imgReg.get(getImageName()) == null) {
			imgReg.put(getImageName(), image);
		}
		if (imgReg.get(getThumbImageName()) == null) {
			imgReg.put(getThumbImageName(), thumb);
		}
		if (imgReg.get(getActiveThumbName()) == null) {
			imgReg.put(getActiveThumbName(),act_thumb);
		}

		setConstraint(xoff, bound.y, 48, 48);
		int end = viewer.getTimeLine().getTimeOfOffset(xoff + 48);
		model.setEnd(end,end);
	}

	@Override
	protected void paintFigure(Graphics g2d) {
		Rectangle clArea = getClientArea();

		Image img;
		if( getActive() )
		{
			img = imgReg.get(getActiveThumbName());
		}
		else
		{
			img = imgReg.get(getThumbImageName());
		}
		if (img != null)
			g2d.drawImage(img, new Point(clArea.x,
					clArea.y));
	}

	/**
	 * @return Returns the model.
	 */
	@Override
	public ImageModel getModel() {
		return model;
	}

	public void propertyChange(PropertyChangeEvent evt) {
		if (evt.getPropertyName().equals(AbstractModel.P_SELECT)) {
			viewer.getImageViewer().setModel(model);
		} else if (evt.getPropertyName().equals(AbstractModel.P_DESELECT)) {
			if (viewer.getImageViewer().getImageModel() == model) {
				viewer.getImageViewer().setModel(null);
			}
		}
	}

	@Override
	public String getID() {
		return model.getID();
	}

	@Override
	public void updateColors() {
		// nothing
	}
}